#include <regex>

#include "Game.h"
#include "Field.h"
#include "base.h"
#include "pieces/King.h"

using namespace std;

CGame::CGame() {
  m_CurrentPlayer = Color::WHITE;
  m_pField = new CField();
}

CKing *CGame::getActiveKing() {
  return (m_CurrentPlayer == Color::WHITE ? m_pField->getKingWhite() : m_pField->getKingBlack());
}

bool CGame::parsePosition(string pos, int *x, int *y) {

  if(pos.length() != 2)
    return false;

  int first = pos[0] - (pos[0] >= (int)('a') ? (int)('a') : (int)('A'));
  int second = pos[1] - (int)('1');

  if(first < 0 || first >= 8)
    return false;

  if(second < 0 || second >= 8)
    return false;

  *x = first;
  *y = second;
  return true;
}

bool CGame::processInput(string line) {
  trim(line);

  if(line.compare("q") == 0 || line.compare("quit") == 0 || line.compare("exit") == 0) {
    m_IsOver = true;
    return true;
  } else if(line.compare("0-0") == 0 || line.compare("O-O") == 0) {
    CKing *activeKing = getActiveKing();
    if(!activeKing->performKingsideCasteling(m_pField)) {
      cout << "Your king can't castle kingside right now." << endl;
      return false;
    } else {
      return true;
    }
  } else if(line.compare("0-0-0") == 0 || line.compare("O-O-O") == 0) {
    CKing *activeKing = getActiveKing();
    if(!activeKing->performQueensideCasteling(m_pField)) {
      cout << "Your king can't castle queenside right now." << endl;
      return false;
    } else {
      return true;
    }
  } else {
    regex rgx("^(.*)-(.*)$");
    smatch match;
    if (regex_search(line, match, rgx)) {
      int fromX, fromY, toX, toY;
      if(parsePosition(match[1].str(), &fromX, &fromY) && parsePosition(match[2].str(), &toX, &toY)) {
        CPiece *piece = m_pField->getPiece(fromX, fromY);
        if(piece == nullptr) {
          cout << "There is no piece on the source field" << endl;
          return false;
        }

        if(piece->getColor() != m_CurrentPlayer) {
          cout << "You can only move your own pieces" << endl;
          return false;
        }

        return (m_pField->move(fromX, fromY, toX, toY));
      }
    }
  }

  cout << "Invalid Input!" << endl;
  return false;
}

void CGame::start() {

  m_IsOver = false;
  m_pField->reset();
  string line;

  while(!m_IsOver) {

    CKing *activeKing = getActiveKing();
    if(m_pField->isAttacked(activeKing->m_X, activeKing->m_Y, 1 - m_CurrentPlayer))
      cout << "Check!" << endl;

    cout << endl << *m_pField << endl;

    do {
      cout << "[" << (m_CurrentPlayer == Color::WHITE ? "WHITE" : "BLACK") << " Turn]: ";
      cin >> line;
    } while(!processInput(line));

    // TODO: check win condition
    {
      m_CurrentPlayer = 1 - m_CurrentPlayer;
    }
  }
}

int main() {
  CGame game;
  game.start();
}
