#include "King.h"
#include "../Field.h"

CKing::CKing(int x, int y, int color)
  : CPiece(Type::KING, (color == Color::WHITE ? "♔" : "♚"), x, y, color) {
}

bool CKing::performCasteling(CField *pField, int rookPos) {

  // 1. King may not have been moved yet
  if(m_HasMoved)
    return false;

  // 2. Rook may not have been moved yet
  CPiece *piece = pField->getPiece(rookPos, m_Y);
  if(piece == nullptr || piece->getType() != Type::ROOK || piece->m_HasMoved)
    return false;


  for(int x = m_X; (rookPos == 0 ? x >= 0 : x <= 7); (rookPos == 0 ? x-- : x++)) {

    // 3. Empty fields between king and rook
    if(x != rookPos && x != m_X) {
      if(pField->getPiece(x, m_Y) != nullptr) {
        return false;
      }
    }

    // 4. No fields are attacked
    if(pField->isAttacked(x, m_Y, 1 - getColor())) {
      return false;
    }
  }

  // Swap!
  pField->setPiece(m_X, m_Y, piece);
  pField->setPiece(rookPos, m_Y, piece);
  m_HasMoved = true;
  piece->m_HasMoved = true;
  return true;
}

bool CKing::canMove(CField *pField, int destX, int destY) {

  int dirX = destX - m_X;
  int dirY = destY - m_Y;

  if(dirY == 0 && (destX == 0 || destX == 7))
    return performCasteling(pField, destX);

  if(abs(dirX) > 1 || abs(dirY) > 1)
    return false;

  if(pField->isAttacked(destX, destY, 1 - getColor()))
    return false;

  return true;
}
